@@ -227,7 +227,7 @@ class Tree:
                                 {"kind": "record", "name": branch_name, "keys": keys}
                             )
 
-                            for key, cont in zip(keys, contents):
+                            for key, cont in zip(keys, contents, strict=True):
                                 subname = self._field_name(branch_name, key)
                                 dtype = self._branch_ak_to_np(cont)
                                 if dtype is None:
@@ -273,7 +273,7 @@ class Tree:
                             {"kind": "record", "name": branch_name, "keys": keys}
                         )
 
-                        for key, content in zip(keys, contents):
+                        for key, content in zip(keys, contents, strict=True):
                             subname = self._field_name(branch_name, key)
                             dtype = self._branch_ak_to_np(content)
                             if dtype is None:
@@ -462,7 +462,7 @@ class Tree:
         if self._num_baskets >= self._basket_capacity - 1:
             self._basket_capacity = max(
                 self._basket_capacity + 1,
-                int(math.ceil(self._basket_capacity * self._resize_factor)),
+                math.ceil(self._basket_capacity * self._resize_factor),
             )
 
             for datum in self._branch_data:
@@ -525,7 +525,7 @@ class Tree:
                     }
                 else:
                     provided = {}
-                for k, v in zip(awkward.fields(data), awkward.unzip(data)):
+                for k, v in zip(awkward.fields(data), awkward.unzip(data), strict=True):
                     provided[k] = v
 
         if isinstance(data, numpy.ndarray) and data.dtype.fields is not None:
@@ -735,7 +735,7 @@ class Tree:
                             )
                         )
                     tofill.append((branch_name, datum["compression"], big_endian, None))
-                    if datum["kind"] == "counter":
+                    if datum["kind"] == "counter" and big_endian.size > 0:
                         datum["tleaf_maximum_value"] = max(
                             big_endian.max(), datum["tleaf_maximum_value"]
                         )
@@ -1395,6 +1395,13 @@ class Tree:
         fObjlen = len(uncompressed_data)
         fNbytes = fKeylen + len(compressed_data)
 
+        if max(fObjlen, fNbytes) > uproot.const.kMaxTBasketBytes:
+            raise ValueError(
+                f"Numpy array data of branch {branch_name} has an uncompressed size of {fObjlen} bytes "
+                f"and a compressed size of {fNbytes} bytes, which is too large to fit in a TBasket. "
+                "Neither of these sizes can exceed 2 GiB."
+            )
+
         parent_location = self._directory.key.location  # FIXME: is this correct?
 
         location = self._freesegments.allocate(fNbytes, dry_run=False)
@@ -1470,6 +1477,13 @@ class Tree:
         fObjlen = len(uncompressed_data)
         fNbytes = fKeylen + len(compressed_data)
 
+        if max(fObjlen, fNbytes) > uproot.const.kMaxTBasketBytes:
+            raise ValueError(
+                f"Jagged array data of branch {branch_name} has an uncompressed size of {fObjlen} bytes "
+                f"and a compressed size of {fNbytes} bytes, which is too large to fit in a TBasket. "
+                "Neither of these sizes can exceed 2 GiB."
+            )
+
         parent_location = self._directory.key.location  # FIXME: is this correct?
 
         location = self._freesegments.allocate(fNbytes, dry_run=False)
@@ -1559,6 +1573,13 @@ class Tree:
         fObjlen = len(uncompressed_data)
         fNbytes = fKeylen + len(compressed_data)
 
+        if max(fObjlen, fNbytes) > uproot.const.kMaxTBasketBytes:
+            raise ValueError(
+                f"String data of branch {branch_name} has an uncompressed size of {fObjlen} bytes "
+                f"and a compressed size of {fNbytes} bytes, which is too large to fit in a TBasket. "
+                "Neither of these sizes can exceed 2 GiB."
+            )
+
         parent_location = self._directory.key.location  # FIXME: is this correct?
 
         location = self._freesegments.allocate(fNbytes, dry_run=False)
